package com.softbankrobotics.pepperhelloworld;

import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.Button;

import com.aldebaran.qi.Future;
import com.aldebaran.qi.sdk.QiContext;
import com.aldebaran.qi.sdk.QiSDK;
import com.aldebaran.qi.sdk.RobotLifecycleCallbacks;
import com.aldebaran.qi.sdk.builder.AnimateBuilder;
import com.aldebaran.qi.sdk.builder.AnimationBuilder;
import com.aldebaran.qi.sdk.builder.SayBuilder;
import com.aldebaran.qi.sdk.design.activity.RobotActivity;
import com.aldebaran.qi.sdk.object.actuation.Animate;
import com.aldebaran.qi.sdk.object.actuation.Animation;
import com.aldebaran.qi.sdk.object.conversation.Say;
import com.aldebaran.qi.sdk.object.human.Human;
import com.aldebaran.qi.sdk.object.humanawareness.HumanAwareness;
import com.aldebaran.qi.sdk.object.touch.Touch;
import com.aldebaran.qi.sdk.object.touch.TouchSensor;

import java.util.List;
import java.util.concurrent.atomic.AtomicBoolean;

public class ChainingActionActivity extends RobotActivity implements RobotLifecycleCallbacks {

    private Button startChainButton;
    private Say helloSay;
    private Animation helloAnimation;
    private Say introductionSay;
    private Animate helloAnimate;
    private Animate animate;
    private Say endingSay;
    private Button stopChainButton;
    private Future<Void> chainFuture;
    private Button skipHelloButton;
    private Future<Void> helloFuture;
    private TouchSensor touchSensor;
    private HumanAwareness humanAwareness;
    private AtomicBoolean isChainRunning = new AtomicBoolean(false);

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_chaining_action);

        startChainButton = findViewById(R.id.startChainButton);
        startChainButton.setOnClickListener(view -> {
            startChainAction();
        });

        stopChainButton = findViewById(R.id.stopChainButton);
        stopChainButton.setOnClickListener(view -> {
            stopChainAction();
        });


        skipHelloButton = findViewById(R.id.skipHelloButton);
        skipHelloButton.setOnClickListener(view -> {
            skipHelloAction();
        });

        QiSDK.register(this, this);

    }

    private void skipHelloAction() {
        if(helloFuture != null && !helloFuture.isDone()){
            helloFuture.requestCancellation();
        }
    }

    private void stopChainAction() {
        if(chainFuture != null && !chainFuture.isDone()){
            chainFuture.requestCancellation();
        }
    }

    private void startChainAction() {

        if(isChainRunning.compareAndSet(false, true)) {
            Log.i("MyTag","The hello future has ended with success");

            Future<Void> helloSayFuture = helloSay.async().run();
            Future<Void> helloAnimateFuture = helloAnimate.async().run();

            helloFuture = Future.waitAll(helloSayFuture, helloAnimateFuture);

            chainFuture = helloFuture
                    .thenCompose(helloFutureResult -> {
                        if (helloFutureResult.isSuccess()) {
                            Log.i("MyTag", "The hello future has ended with success");
                        } else if (helloFutureResult.isCancelled()) {
                            Log.i("MyTag", "The hello future has been cancelled");
                        } else if (helloFutureResult.hasError()) {
                            Log.i("MyTag", "The hello future has finished with an error; "
                                    + helloFutureResult.getErrorMessage());
                        }

                        return introductionSay.async().run();

                    })
                    .andThenCompose(aVoid -> animate.async().run())
                    .andThenCompose(aVoid -> endingSay.async().run());

            chainFuture.thenConsume(voidFuture -> {

                isChainRunning.set(false);
                Log.i("MyTag", "The chain is stopped");

                if (voidFuture.hasError()) {
                    Log.i("MyTag", "The chain finish with error: " + voidFuture.getErrorMessage());
                } else {
                    Log.i("MyTag", "The chain finish without error: ");
                }
            });
        }
    }

    @Override
    protected void onDestroy() {

        QiSDK.unregister(this,this);

        super.onDestroy();
    }

    @Override
    public void onRobotFocusGained(QiContext qiContext) {

        humanAwareness = qiContext.getHumanAwareness();
        humanAwareness.addOnHumansAroundChangedListener(humans -> {
            Log.i("MyTag","I detected " + humans.size() + " aropund me.");

            if(humans.size() > 0){

                Human human = humans.get(0);
                Log.i("MyTag", "Estimated age: " + human.getEstimatedAge().getYears());
                Log.i("MyTag", "Estimated gender: " + human.getEstimatedGender());
                Log.i("MyTag", "Head frame: " + human.getHeadFrame());

                startChainAction();
            }
        });

        Touch touch = qiContext.getTouch();
        touchSensor = touch.getSensor("Head/Touch");
        touchSensor.addOnStateChangedListener(state -> {
            Log.i("MyTag","The touch state has changed to " + state.getTouched());
            if(state.getTouched()){
                stopChainAction();
            }
        });

        helloSay = SayBuilder.with(qiContext)
                .withText("こんにちは　ペッパーです　皆さん　お元気ですか？")
                .build();

        helloAnimation = AnimationBuilder.with(qiContext)
                .withResources(R.raw.hello_a001)
                .build();

        helloAnimate = AnimateBuilder.with(qiContext)
                .withAnimation(helloAnimation)
                .build();


        introductionSay = SayBuilder.with(qiContext)
                .withText("今からダンスを踊ります　見ていてください")
                .build();

        Animation animation = AnimationBuilder.with(qiContext)
                .withResources(R.raw.disco_a001)
                .build();

        animate = AnimateBuilder.with(qiContext)
                .withAnimation(animation)
                .build();

        endingSay = SayBuilder.with(qiContext)
                .withText("いかがでしたか？")
                .build();


    }

    @Override
    public void onRobotFocusLost() {
        if(touchSensor != null){
            touchSensor.removeAllOnStateChangedListeners();
        }

        if(humanAwareness != null){
            humanAwareness.removeAllOnHumansAroundChangedListeners();
        }
    }

    @Override
    public void onRobotFocusRefused(String reason) {

    }
}
