package com.softbankrobotics.pepperhelloworld;

import android.graphics.Color;
import android.support.constraint.ConstraintLayout;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.Button;

import com.aldebaran.qi.Consumer;
import com.aldebaran.qi.sdk.QiContext;
import com.aldebaran.qi.sdk.QiSDK;
import com.aldebaran.qi.sdk.RobotLifecycleCallbacks;
import com.aldebaran.qi.sdk.builder.AnimateBuilder;
import com.aldebaran.qi.sdk.builder.AnimationBuilder;
import com.aldebaran.qi.sdk.builder.ChatBuilder;
import com.aldebaran.qi.sdk.builder.QiChatbotBuilder;
import com.aldebaran.qi.sdk.builder.SayBuilder;
import com.aldebaran.qi.sdk.builder.TopicBuilder;
import com.aldebaran.qi.sdk.design.activity.RobotActivity;
import com.aldebaran.qi.sdk.object.actuation.Animate;
import com.aldebaran.qi.sdk.object.actuation.Animation;
import com.aldebaran.qi.sdk.object.conversation.AutonomousReactionImportance;
import com.aldebaran.qi.sdk.object.conversation.AutonomousReactionValidity;
import com.aldebaran.qi.sdk.object.conversation.BaseQiChatExecutor;
import com.aldebaran.qi.sdk.object.conversation.Bookmark;
import com.aldebaran.qi.sdk.object.conversation.Chat;
import com.aldebaran.qi.sdk.object.conversation.QiChatExecutor;
import com.aldebaran.qi.sdk.object.conversation.QiChatVariable;
import com.aldebaran.qi.sdk.object.conversation.QiChatbot;
import com.aldebaran.qi.sdk.object.conversation.Say;
import com.aldebaran.qi.sdk.object.conversation.Topic;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Random;

public class MainActivity extends RobotActivity implements RobotLifecycleCallbacks {

    private QiChatbot qiChatbot;
    private ConstraintLayout mainFrame;
    private Button showColorButton;
    private Topic updateColorTopic;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        mainFrame = findViewById(R.id.mainFrame);

        showColorButton = findViewById(R.id.showColorButton);
        showColorButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {

                if(updateColorTopic == null || qiChatbot == null){
                    return;
                }

                updateColorTopic.async().getBookmarks()
                        .andThenConsume(new Consumer<Map<String, Bookmark>>() {
                            @Override
                            public void consume(Map<String, Bookmark> stringBookmarkMap) throws Throwable {
                                Bookmark bookmark = stringBookmarkMap.get("button_clicked");

                                qiChatbot.goToBookmark(bookmark,
                                        AutonomousReactionImportance.HIGH,
                                        AutonomousReactionValidity.IMMEDIATE);
                            }
                        });
                updateTabletColor();
            }
        });

        QiSDK.register(this,this);
    }

    @Override
    protected void onDestroy() {

        QiSDK.unregister(this,this);

        super.onDestroy();
    }

    @Override
    public void onRobotFocusGained(QiContext qiContext) {
        Log.i("MyTag", "onRobotFocusGained");

        /*Say say = SayBuilder.with(qiContext)
                .withText("こんにちは　ペッパーです")
                .build();

        say.run();*/

        Topic sayHelloTopic = TopicBuilder.with(qiContext)
                .withResource(R.raw.hello_topic)
                .build();

        updateColorTopic = TopicBuilder.with(qiContext)
                .withResource(R.raw.update_color)
                .build();

        qiChatbot = QiChatbotBuilder.with(qiContext)
                .withTopic(sayHelloTopic, updateColorTopic)
                .build();

        qiChatbot.addOnBookmarkReachedListener(new QiChatbot.OnBookmarkReachedListener() {
            @Override
            public void onBookmarkReached(Bookmark bookmark) {
                if("show_favorite_color".equals(bookmark.getName())){
                    updateTabletColor();
                }
            }
        });

        Chat chat = ChatBuilder.with(qiContext)
                .withChatbot(qiChatbot)
                .build();

        Map<String, QiChatExecutor> executors = new HashMap<>();
        executors.put("randomColorExecutor", new RandomColorExecutor(qiContext));
        qiChatbot.setExecutors(executors);

        chat.async().run();

    }

    private void updateTabletColor() {
        if(qiChatbot == null){
            return;
        }
        //String tabletColor = qiChatbot.variable(("favorite_color")).getValue();
        qiChatbot.async().variable("favorite_color")
                .andThenConsume(new Consumer<QiChatVariable>() {
                    @Override
                    public void consume(QiChatVariable qiChatVariable) throws Throwable {
                        String tabletColor = qiChatVariable.getValue();
                        final int androidTabletColor = getAndroidTabletColor(tabletColor);

                        displayColor(androidTabletColor);
                    }
                });
    }

    private void displayColor(final int androidTabletColor) {
        runOnUiThread(new Runnable() {
            @Override
            public void run() {
                mainFrame.setBackgroundColor(androidTabletColor);
            }
        });
    }

    private int getAndroidTabletColor(String color) {
        switch (color){
            case "赤":
                return Color.RED;
            case "青":
                return Color.BLUE;
            case "緑":
                return Color.GREEN;
            default:
                return 0;
        }
    }


    @Override
    public void onRobotFocusLost() {

    }

    @Override
    public void onRobotFocusRefused(String reason) {

    }

    class RandomColorExecutor extends BaseQiChatExecutor {
        protected RandomColorExecutor(QiContext context) {
            super(context);
        }

        @Override
        public void runWith(List<String> params) {
            boolean bool = new Random().nextBoolean();

            int color;

            if(bool) {
                color = Color.BLUE;
            }
            else {
                color = Color.RED;
            }

            Animation animation = AnimationBuilder.with(getQiContext())
                    .withResources(R.raw.gesture_both_hands_b001)
                    .build();

            Animate animate = AnimateBuilder.with(getQiContext())
                    .withAnimation(animation)
                    .build();

            animate.run();

            displayColor(color);
        }

        @Override
        public void stop() {

        }
    }
}
